#ifndef __XP_STRUCT_H__
#define __XP_STRUCT_H__

/*
* © Carwyn Jones (2002) turok2@currantbun.com
* A simple driver to enable SSE instructions on the Athlon XP processors(BeOS)
* Uses code from various developers in the Linux community.
*/
#define SSEENABLE 'si'
#define SSEDISABLE 'di'


#define MSR_K7_HWCR 0xC0010015
#define X86_FEATURE_XMM	(0*32+25)
#define NCAPINTS 4
#ifdef CONFIG_SMP
#define LOCK_PREFIX "lock ; "
#else
#define LOCK_PREFIX ""
#endif
#define X86_CR4_OSFXSR		0x0200
#define X86_CR4_OSXMMEXCPT	0x0400

#define ADDR (*(volatile long *) addr)

#define rdmsr(msr,val1,val2) \
     __asm__ __volatile__("rdmsr" \
                          : "=a" (val1), "=d" (val2) \
                          : "c" (msr))

#define wrmsr(msr,val1,val2) \
     __asm__ __volatile__("wrmsr" \
                          : /* no outputs */ \
                          : "c" (msr), "a" (val1), "d" (val2))


static inline int constant_test_bit(int nr, const volatile void * addr)
{
	return ((1UL << (nr & 31)) & (((const volatile unsigned int *) addr)[nr >> 5])) != 0;
}

static inline int variable_test_bit(int nr, volatile void * addr)
{
	int oldbit;

	__asm__ __volatile__(
		"btl %2,%1\n\tsbbl %0,%0"
		:"=r" (oldbit)
		:"m" (ADDR),"Ir" (nr));
	return oldbit;
}

#define test_bit(nr,addr) \
(__builtin_constant_p(nr) ? \
 constant_test_bit((nr),(addr)) : \
 variable_test_bit((nr),(addr)))
 
 
unsigned long x86_capability[NCAPINTS];

static inline void set_bit(int nr, volatile void * addr)
{
	__asm__ __volatile__( LOCK_PREFIX
		"btsl %1,%0"
		:"=m" (ADDR)
		:"Ir" (nr));
}

static inline void set_in_rdmsr(unsigned long msr, unsigned long *val1, unsigned long *val2)
{
	__asm__ __volatile__("rdmsr"
                          : "=a" (*val1), "=d" (*val2)
                          : "c" (msr));
}

static inline void set_in_wrmsr(unsigned long msr, unsigned long val1, unsigned long val2)
{
	__asm__ __volatile__("wrmsr"
                          : /* no outputs */
                          : "c" (msr), "a" (val1), "d" (val2));
}

static inline void set_in_cr4 (unsigned long mask)
{
	__asm__("movl %%cr4,%%eax\n\t"
		"orl %0,%%eax\n\t"
		"movl %%eax,%%cr4\n"
		: : "irg" (mask)
		:"ax");
}

static inline void clear_in_cr4 (unsigned long mask)
{
	__asm__("movl %%cr4,%%eax\n\t"
		"andl %0,%%eax\n\t"
		"movl %%eax,%%cr4\n"
		: : "irg" (~mask)
		:"ax");
}

static void EnableXpSSE();
static void DisableXpSSE();

	
#endif